# ////////////////////////////////////////////////////////////////////////////
# //
# //  D SOFTWARE INCORPORATED
# //  Copyright 2007-2014 D Software Incorporated
# //  All Rights Reserved.
# //
# //  NOTICE: D Software permits you to use, modify, and distribute this file
# //  in accordance with the terms of the license agreement accompanying it.
# //
# //  Unless required by applicable law or agreed to in writing, software
# //  distributed under the License is distributed on an "AS IS" BASIS,
# //  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# //
# ////////////////////////////////////////////////////////////////////////////
#
# This is a sample of what can be done by using API with Zephyr for JIRA through the Python coding language.
# The goal of the sample is to create a new test cycle and add existing test issues into it
# 
# IDLE IDE - Version: 2.7.5
# Python - Version: 2.7.5
# 
# Author: Daniel Gannon, Technical Support Analyst, D Software Inc.

# Import 'csv' and from libraries 'urllib2', 'urllib', 'json', 'base64', 'collections'
from base64 import b64encode
from urllib2 import Request, urlopen
from urllib import urlencode
from json import load
from collections import Counter
import csv

# Variables that can be modified:
username = 'test.manager'
password = 'test.manager'
projectId = 10000
versionId = 10200
filename = 'cycleExecutionReport.csv'

# 'baseURL' holds basic data for JIRA server that will be used in API URL calls
baseURL = 'http://localhost:8080'
getPVInfoURL = baseURL + "/rest/zapi/latest/util/versionBoard-list"
getCyclesURL = baseURL + "/rest/zapi/latest/cycle"
getExecutionsURLBase = baseURL + "/rest/zapi/latest/execution"

# 'headers' holds information to be sent with the JSON data set
# Initialized with Auth and Content-Type data
# Authorization header uses base64 encoded username and password string
headers = {"Authorization": " Basic " + b64encode(username + ":" + password), "Content-Type": "application/json"}

# ///// Get Project & Version Label for Header Purposes /////
# Quick fetch for Version and Project names based on provided ID's to be used in CSV report

print "Compiling CSV File Header Information..."

pvInfoValues = {
    "projectId": projectId,
    "versionId": versionId,
    "showUnscheduled": 'false'
}

getPVInfoURL = getPVInfoURL + "?" + urlencode(pvInfoValues)

request = Request(getPVInfoURL, None, headers=headers)
js_res = urlopen(request)
fetchedVersions = load(js_res)

projectName = fetchedVersions['project']['name']
versionName = fetchedVersions['version']['name']

print "Completed Header Information Compile!\n"

# ///// Get All Test Cycles by Project and Version /////
# Based on and project and version in that project, search for all cycles
# Returned cycle have their ID's recorded for use later on

print "Compiling Cycle Information for Execution Search..."

cycleValues = {
    "projectId": projectId,
    "versionId": versionId
}

getCyclesURL = getCyclesURL + "?" + urlencode(cycleValues)

request = Request(getCyclesURL, None, headers=headers)
js_res = urlopen(request)
fetchedCycles = load(js_res)

totalCycles = fetchedCycles.pop(u'recordsCount')

print "Completed Cycle Information Compile!\n"

# ///// Get Executions and Write to CSV /////
# Fetch all test executions inside of the cycle ID's returned from above
# CSV file is created. Writing process happens each time a cycle is looped through
# Two 'Counter()' sets are used to tally current cycle amounts and total amounts

print "Creating CSV File to Write..."

with open(filename, "wb") as outfile:
    f = csv.writer(outfile)
    f.writerow(["Cycle Execution Report By Version and Project"])
    f.writerow(["PROJECT:", projectName])
    f.writerow(["VERSION:", versionName])
    f.writerow([""])

print "Completed CSV File Creation!\n"

cycleSet = Counter()
totalCycleSet = Counter()
for key, value in iter(sorted(fetchedCycles.iteritems())):
    print "Compiling Executions and Writing to CSV File...Cycle: " + str(key) + ", " + str(value['name'])
    
    exValues = {
        "cycleId": key,
        "projectId": projectId,
        "versionId": versionId,
        "sorter": "ID: DESC"
    }

    getExecutionsURL = getExecutionsURLBase + "?" + urlencode(exValues)

    request = Request(getExecutionsURL, None, headers=headers)
    js_res = urlopen(request)
    fetchedExecutions = load(js_res)

    cycleExList = fetchedExecutions['executions']
        
    cycleSet.clear()
    for x in cycleExList:                
        cycleSet[str(x['executionStatus'])] += 1

    with open(filename, "ab") as outfile:
        f = csv.writer(outfile)
        f.writerow(["CYCLE:", str(value['name'])])
        f.writerow(["", "", "PASS:", cycleSet['1']])
        f.writerow(["", "", "FAIL:", cycleSet['2']])
        f.writerow(["", "", "WIP:", cycleSet['3']])
        f.writerow(["", "", "BLOCKED:", cycleSet['4']])
        f.writerow(["", "", "UNEXECUTED:", cycleSet['-1']])

    totalCycleSet = cycleSet + totalCycleSet

# ///// Total Report Numbers and End Writing Process /////
# Finishes writing to CSV with total amounts gathered from above

print "\nTotaling Report Numbers and Writing to CSV file..."

with open(filename, "ab") as outfile:
    f = csv.writer(outfile)
    f.writerow([""])
    f.writerow(["TOTAL CYCLES:", totalCycles])
    f.writerow(["TOTAL EXECUTIONS:", (totalCycleSet['1'] + totalCycleSet['2'] + totalCycleSet['3'] + totalCycleSet['4'])])
    f.writerow(["TOTAL ASSIGNED:", (totalCycleSet['1'] + totalCycleSet['2'] + totalCycleSet['3'] + totalCycleSet['4'] + totalCycleSet['-1'])])

print "Completed CSV Report!"
